----------------------------------------------------------------------------
-- You Looted A Quest Starter Item
-- Copyright (c) 2009-2016 Rikard Glans (natch@brainlag.se)
----------------------------------------------------------------------------
-- Time-stamp: <2016-08-05 01:49:11>
----------------------------------------------------------------------------
-- Frame setup
local YI = CreateFrame("Frame", "YouLootedAQuestStarterItemFrame", UIParent);
YI:SetScript("OnUpdate", function(_, elapsed) YI["OnUpdate"](YI, elapsed) end);
YI:SetScript("OnEvent", function(_, event, ...) YI[event](YI, ...) end);
YI:RegisterEvent("ADDON_LOADED");
YI:RegisterEvent("VARIABLES_LOADED");
YI:RegisterEvent("BAG_UPDATE");
YI:RegisterEvent("UNIT_INVENTORY_CHANGED");
local TT = CreateFrame("GameTooltip", "YouLootedAQuestStarterItemTooltip", UIParent, "GameTooltipTemplate");
----------------------------------------------------------------------------
-- Constants
local TT_TEXT = "This Item Begins a Quest"; -- TODO: Support all languages.

local onoff = {
  [true] = "Enabled",
  [false] = "Disabled"
};
----------------------------------------------------------------------------
-- Variables
local ndebug = false;
local player = nil;
----------------------------------------------------------------------------
function YI:OnUpdate(elapsed)
  self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed;

  if(self.TimeSinceLastUpdate > self.UpdateInterval) then
    -- self:print("DEBUG: OnUpdate()");

    local bagid = 0;
    local qs = self:QueueSize();
    -- self:debug("qs: " .. qs)
    for bagid = 0, qs do
      local bagid = self:QueuePop();

      if(bagid) then
        self:ScanBag(bagid);
      end
    end

    self.TimeSinceLastUpdate = 0;
  end
end

-- Events
function YI:ADDON_LOADED(msg)
  if(msg == "Natch_YouLootedAQuestStarterItem") then
    self:UnregisterEvent("ADDON_LOADED");

    _G.SLASH_NATCH_YOULOOTEDAQUESTSTARTERITEM1          = "/nyi";
    _G.SLASH_NATCH_YOULOOTEDAQUESTSTARTERITEM2          = "/youlootedaqueststarteritem";
    _G.SlashCmdList["NATCH_YOULOOTEDAQUESTSTARTERITEM"] = function(...) self:SlashCMD(...); end;

    player = UnitName("player").."@"..GetRealmName();

    YI.Queue               = {};
    YI.UpdateInterval      = 2;
    YI.TimeSinceLastUpdate = 0;

    self:print("You Looted A Quest Starter Item loaded!");

    self.ADDON_LOADED = nil;
  end
end

function YI:VARIABLES_LOADED()
  self:UnregisterEvent("VARIABLES_LOADED");

  if(not YI_Data) then
    YI_Data = {
      [player] = {
        ["warnCache"] = {};
      },
      ["annoying"] = false,
      ["ultraannoying"] = false;
    };
  end

  if(not YI_Data[player]) then
    YI_Data[player] = {
      ["warnCache"] = {};
    };
  end

  -- Alpha 002 update
  if(not YI_Data["ultraannoying"]) then
    YI_Data["ultraannoying"] = false;
  end

  self.VARIABLES_LOADED = nil;
end

function YI:BAG_UPDATE(bagid)
  local bagid = bagid;

  if(bagid) then
    self:QueuePush(bagid);
  end
end

function YI:UNIT_INVENTORY_CHANGED(unitid)
  local bag = 0;

  for bag = 0, NUM_BAG_SLOTS do
    self:QueuePush(bag);
  end
end

----------------------------------------------------------------------------
-- Functions

-- Put a bag in the queue
function YI:QueuePush(bagid)
  if(not self:QueueExists(bagid)) then
    table.insert(self.Queue, bagid);
  end
end

-- Remove and return last item from the stack
function YI:QueuePop()
  local bagid = table.remove(self.Queue);
  return(bagid);
end

-- Check if bagid is already in queue
function YI:QueueExists(bagid)
  for i, qid in ipairs(self.Queue) do
    if(qid == bagid) then
      return(true);
    end
  end

  return(false);
end

-- Return number of bags in queue
function YI:QueueSize()
  return(#self.Queue);
end

-- Calls ScanTT on all items in bag with id <bagid>
function YI:ScanBag(bagid)
  local bagid = bagid;

  self:debug("Scanning bagid: " .. bagid);

  for slot = 1, GetContainerNumSlots(bagid) do
    local itemLink = GetContainerItemLink(bagid, slot);

    if(itemLink) then
      self:ScanTT(itemLink);
    end
  end
end

-- Scans tooltip for TT_TEXT
function YI:ScanTT(link)
  local link = link;

  -- self:debug("link = \"" .. string.gsub(link, "\124", "\124\124") .. "\"");

  if(string.find(link, "battlepet")) then
    -- self:debug("Found battlepet, skipping.");

    return;
  end

  TT:SetOwner(WorldFrame, "ANCHOR_NONE");
  GameTooltip_SetDefaultAnchor(TT, UIParent);
  TT:SetHyperlink(link);

  for i = 1, TT:NumLines() do
    local text = strlower(getglobal("YouLootedAQuestStarterItemTooltipTextLeft" .. i):GetText());

    -- self:debug("text = " .. text);

    if(text and text:find(strlower(TT_TEXT))) then
      if(not self:InCache(link)) then
        self:debug("Found quest starter item: " .. link);

        local str = link .. " starts a quest!";
        self:Warn(str);
        self:Cache(link);
      end
    end
  end

  TT:Hide();
end

-- Updates cache with new item.
function YI:Cache(link)
  local link = link;
  local _, _, itemID = string.find(link, "|c%x+|Hitem:(%d+):");

  self:debug("Cache(): itemID: " .. itemID);

  YI_Data[player]["warnCache"][itemID] = true;
end

-- Returns true if item exists in cache, otherwise false.
function YI:InCache(link)
  local link = link;
  local _, _, itemID = string.find(link, "|c%x+|Hitem:(%d+):");

  if(YI_Data[player]["warnCache"][itemID]) then
    -- self:debug("InCache(): itemID: " .. itemID);

    return(true);
  end

  return(false);
end

-- Print warning with desired method.
function YI:Warn(str)
  local msg = str;

  if(MikSBT ~= nil) then
    if(YI_Data.annoying == true) then
      for scrollAreaKey, _ in MikSBT.IterateScrollAreas() do
        if(YI_Data.ultraannoying == true) then
          MikSBT.DisplayMessage(msg, scrollAreaKey);
          MikSBT.DisplayMessage(msg, scrollAreaKey);
          MikSBT.DisplayMessage(msg, scrollAreaKey);
          MikSBT.DisplayMessage(msg, scrollAreaKey);
        end

        MikSBT.DisplayMessage(msg, scrollAreaKey);
      end

      if(YI_Data.ultraannoying == true) then
        self:print(msg);
        self:print(msg);
        self:print(msg);
        self:print(msg);
      end

      self:print(msg);
    else
      MikSBT.DisplayMessage(msg);

      self:print(msg);
    end

    return;
  else
    if(YI_Data.ultraannoying == true) then
      UIErrorsFrame:AddMessage(msg, 1, 1, 0);
      UIErrorsFrame:AddMessage(msg, 1, 1, 0);
      UIErrorsFrame:AddMessage(msg, 1, 1, 0);
      UIErrorsFrame:AddMessage(msg, 1, 1, 0);

      self:print(msg);
      self:print(msg);
      self:print(msg);
      self:print(msg);
    end

    UIErrorsFrame:AddMessage(msg, 1, 1, 0);
    self:print(msg);
  end
end

-- Parse command line.
function YI:SlashCMD(str)
  local cmd, arg = CmdArg(string.lower(str));

  if(not cmd or cmd == "") then
    self:PrintHelp();
  elseif(cmd == "help" or cmd == "h") then
    self:PrintHelp();
  elseif(cmd == "a" or cmd == "annoying") then
    if(YI_Data.annoying == true) then
      YI_Data.annoying = false;
      self:print("Annoying mode disabled.");
    else
      YI_Data.annoying = true;
      self:print("Annoying mode enabled.");
    end
  elseif(cmd == "ua" or cmd == "ultraannoying") then
    if(YI_Data.ultraannoying == true) then
      YI_Data.ultraannoying = false;
      self:print("Ultra annoying mode disabled.");
    else
      YI_Data.annoying = true;
      YI_Data.ultraannoying = true;
      self:print("Ultra annoying mode enabled.");
    end
  elseif(cmd == "reset" or cmd == "r") then
    YI_Data[player]["warnCache"] = {};
    self:print("Item cache reset.");
  elseif(cmd == "s" or cmd == "status") then
    self:print("Status:");
    self:print("Annoying mode: " .. onoff[YI_Data["annoying"]]);
    self:print("Ultra annoying mode: " .. onoff[YI_Data["ultraannoying"]]);
  elseif(cmd == "f" or cmd == "force") then
    self:print("Forcing scan of all bags.");

    local bag = 0;
    for bag = 0, NUM_BAG_SLOTS do
      self:QueuePush(bag);
    end
  else
    self:PrintHelp();
  end
end

function YI:PrintHelp()
  self:print("Natch's YouLootedAQuestStarterItem");
  self:print("Usage: /nyi a  - toggle annoying mode");
  self:print("Usage: /nyi ua - toggle ultra annoying mode");
  self:print("Usage: /nyi r  - reset item cache");
end

function YI:print(str, ...)
  SELECTED_CHAT_FRAME:AddMessage("YouLootedAQuestStarterItem: " .. str, ...);
end

function YI:printwol(str, ...)
  SELECTED_CHAT_FRAME:AddMessage(str, ...);
end

function YI:debug(str, ...)
  if(ndebug == true) then
    SELECTED_CHAT_FRAME:AddMessage("YI DEBUG: " .. str, ...);
  end
end

-- Function: CmdArg(string)
-- Usage: local cmd, arg = CmdArg(command-line-string);
function CmdArg(str)
  local argv = {};
  local s,e  = string.find(str, "%s+");

  if(not s or not e) then
    argv[1] = str;
    argv[2] = nil;
  else
    argv[1] = string.sub(str, 1, s-1);
    argv[2] = string.sub(str, e+1, -1);
  end

  return string.lower(argv[1]), argv[2];
end
